%define anolis_release 2
%global debug_package %{nil}

%global toolchain clang

# Opt out of https://fedoraproject.org/wiki/Changes/fno-omit-frame-pointer
# https://bugzilla.redhat.com/show_bug.cgi?id=2158587
%undefine _include_frame_pointers

%bcond_without compat_build

%global maj_ver 18
%global min_ver 1
%global patch_ver 8
%global compiler_rt_version %{maj_ver}.%{min_ver}.%{patch_ver}

%global crt_srcdir compiler-rt-%{compiler_rt_version}.src

%if %{with compat_build}
%global pkg_name compiler-rt%{maj_ver}
%global llvm_pkg_name llvm%{maj_ver}
%else
%global pkg_name compiler-rt
%global llvm_pkg_name llvm
%endif

# see https://sourceware.org/bugzilla/show_bug.cgi?id=25271
%global optflags %(echo %{optflags} -D_DEFAULT_SOURCE)

# see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=93615
%global optflags %(echo %{optflags} -Dasm=__asm__)

Name:		%{pkg_name}
Version:	%{compiler_rt_version}
Release:	%{anolis_release}%{?dist}
Summary:	LLVM "compiler-rt" runtime libraries

License:	Apache-2.0 WITH LLVM-exception OR NCSA OR MIT
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{compiler_rt_version}/%{crt_srcdir}.tar.xz

Patch0:		0001-compiler-rt-Fix-FLOAT16-feature-detection.patch

%if %{with compat_build}
BuildRequires:	clang%{maj_ver}
%else
BuildRequires:	clang
%endif
BuildRequires:	cmake
BuildRequires:	ninja-build
BuildRequires:	python3
# We need python3-devel for %%py3_shebang_fix
BuildRequires:	python3-devel
BuildRequires:	%{llvm_pkg_name}-devel = %{version}
BuildRequires:	%{llvm_pkg_name}-cmake-utils = %{version}
BuildRequires:	zlib-devel

# For gpg source verification
BuildRequires:	gnupg2

%if %{with compat_build}
Requires: clang%{maj_ver}-resource-filesystem = %{version}
Conflicts: compiler-rt < %{lua: return tonumber(macros['maj_ver']) + 1}
%else
Requires: clang-resource-filesystem = %{version}
%endif
Provides: compiler-rt(major) = %{maj_ver}

%description
The compiler-rt project is a part of the LLVM project. It provides
implementation of the low-level target-specific hooks required by
code generation, sanitizer runtimes and profiling library for code
instrumentation, and Blocks C language extension.

%prep
%autosetup -n %{crt_srcdir} -p2

# compiler-rt does not allow configuring LLVM_COMMON_CMAKE_UTILS.
%if %{with compat_build}
ln -s %{_libdir}/llvm%{maj_ver}/share/llvm/cmake ../cmake
%else
ln -s %{_datadir}/llvm/cmake ../cmake
%endif

%py3_shebang_fix lib/hwasan/scripts/hwasan_symbolize

%build
# Copy CFLAGS into ASMFLAGS, so -fcf-protection is used when compiling assembly files.
export ASMFLAGS=$CFLAGS

%cmake	-GNinja \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
%if %{without compat_build}
	-DCMAKE_MODULE_PATH=%{_libdir}/cmake/llvm \
%else
	-DLLVM_CMAKE_DIR="%{_libdir}/llvm%{maj_ver}/lib/cmake/llvm" \
	-DCMAKE_CXX_COMPILER=%{_bindir}/clang++-%{maj_ver} \
	-DCMAKE_C_COMPILER=%{_bindir}/clang-%{maj_ver} \
%endif
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DCOMPILER_RT_INSTALL_PATH=%{_prefix}/lib/clang/%{maj_ver} \
	-DLLVM_ENABLE_PER_TARGET_RUNTIME_DIR=ON \
	-DLLVM_LIBDIR_SUFFIX=64 \
	-DCOMPILER_RT_INCLUDE_TESTS:BOOL=OFF # could be on?

%cmake_build

%install

%cmake_install

%check
#%%cmake_build --target check-compiler-rt

%files
%license LICENSE.TXT
%ifarch x86_64 aarch64 
%{_prefix}/lib/clang/%{maj_ver}/bin/*
%endif
%{_prefix}/lib/clang/%{maj_ver}/include/*
%{_prefix}/lib/clang/%{maj_ver}/lib/*
%{_prefix}/lib/clang/%{maj_ver}/share/*
#%ifarch x86_64 aarch64
#{_bindir}/hwasan_symbolize
#%endif

%changelog
* Thu Mar 27 2025 Chang Gao <gc-taifu@linux.alibaba.com> - 18.1.8-2
- Enable loongarch64 arch due to llvm18 fix .so missing on loongarch64

* Fri Mar 22 2025 Chang Gao <gc-taifu@linux.alibaba.com> - 18.1.8-1
- Import from upstream
- Disable loongarch64 for temporary
